/////////////////////////////////////////////////////////////////////////////
// Project:     wxIView
// Purpose:     Complex wxWidgets sample
// Name:        wxiviewcanvas.h
// Author:      Daniel Nell
// Created:     2005/10/18
// Copyright:   (c) Daniel Nell
// License:     wxWindows license
// Modified by:
/////////////////////////////////////////////////////////////////////////////

#ifndef __WXIVIEWCANVAS_H
#define __WXIVIEWCANVAS_H

/////////////////////////////////////////////////////////////////////////////
//  Constants
//
#define WXIVIEWCANVASDELAY  50


/////////////////////////////////////////////////////////////////////////////
//  ...
//
BEGIN_DECLARE_EVENT_TYPES()
    DECLARE_LOCAL_EVENT_TYPE( wxEVT_IVIEWCANVASSIZE,     0 )
    DECLARE_LOCAL_EVENT_TYPE( wxEVT_IVIEWCANVASSELECTED, 0 )
END_DECLARE_EVENT_TYPES()


/////////////////////////////////////////////////////////////////////////////
//  Class wxIViewCanvasSizeEvent
//
/// \brief The canvas size changing event.
///
/// ...
class wxIViewCanvasSizeEvent : public wxEvent
{
  public:
    /// Constructor
    wxIViewCanvasSizeEvent( wxEventType oType= wxEVT_IVIEWCANVASSIZE )
      : wxEvent( 0, oType )
    {
        // Propagate event to the next event handling window.
        m_propagationLevel= wxEVENT_PROPAGATE_MAX;
    }

    /// Return a copy ot this event.
    wxEvent *Clone() const
    {
        return new wxIViewCanvasSizeEvent( *this );
    }
};

typedef void (wxEvtHandler::*wxIViewCanvasSizeEventEventFunction)( wxIViewCanvasSizeEvent& );

#define wxIViewCanvasSizeEventEventHandler(func) \
    (wxObjectEventFunction)(wxEventFunction)wxStaticCastEvent( wxIViewCanvasSizeEventEventFunction, &func )

#define EVT_IVIEWCANVASSIZE(func) \
   wx__DECLARE_EVT0( wxEVT_IVIEWCANVASSIZE, wxIViewCanvasSizeEventEventHandler( func ) )


/////////////////////////////////////////////////////////////////////////////
//  Class wxIViewCanvasSelectedEvent
//
/// \brief The canvas selection event.
///
/// ...
class wxIViewCanvasSelectedEvent : public wxEvent
{
  public:
    /// Constructor
    wxIViewCanvasSelectedEvent( wxEventType oType= wxEVT_IVIEWCANVASSELECTED )
      : wxEvent( 0, oType ), m_bShift( false ), m_bAlt( false ), m_bCtrl( false ), m_bMeta( false )
    {
        // Propagate event to the next event handling window.
        m_propagationLevel= wxEVENT_PROPAGATE_MAX;
    }

    /// Return a copy ot this event.
    wxEvent *Clone() const
    {
        return new wxIViewCanvasSelectedEvent( *this );
    }

  public:
    bool m_bShift;              ///< Was the SHIFT key pressed while selecting?
    bool m_bAlt;                ///< Was the ALT key pressed while selecting?
    bool m_bCtrl;               ///< Was the CTRL key pressed while selecting?
    bool m_bMeta;               ///< Was the META key pressed while selecting?
    wxRect m_oSelection;        ///< Holds the selection.
};

typedef void (wxEvtHandler::*wxIViewCanvasSelectedEventEventFunction)( wxIViewCanvasSelectedEvent& );

#define wxIViewCanvasSelectedEventEventHandler(func) \
    (wxObjectEventFunction)(wxEventFunction)wxStaticCastEvent( wxIViewCanvasSelectedEventEventFunction, &func )

#define EVT_IVIEWCANVASSELECTED(func) \
   wx__DECLARE_EVT0( wxEVT_IVIEWCANVASSELECTED, wxIViewCanvasSelectedEventEventHandler( func ) )


/////////////////////////////////////////////////////////////////////////////
//  class wxIViewCanvas
//
/// \brief The canvas window.
///
/// ...
class wxIViewCanvas: public wxScrolledWindow
{
  public:
    /// \brief Standard constructor
    ///
    /// \param poParent      a pointer to the dialog windows's parent window.
    /// \param nId           the window id, default: 1
    /// \param oPos          the window position, default: wxDefaultPosition
    /// \param oSize         the window size, default: wxDefaultSize
    /// \param nStyle        window style (default: wxHSCROLL | wxVSCROLL | wxSUNKEN_BORDER)
    /// \param strName       class name (default: wxT( "wxIViewCanvas" ))
    wxIViewCanvas( wxWindow *poParent,
                   wxWindowID nId= wxID_ANY,
                   const wxPoint &oPos= wxDefaultPosition,
                   const wxSize &oSize= wxDefaultSize,
                   long nStyle= wxHSCROLL | wxVSCROLL | wxSUNKEN_BORDER,
                   const wxString& strName= wxT( "wxIViewCanvas" )
                 );

    /// Virtual destructor
    virtual ~wxIViewCanvas(){}

    // Event handlers
    //
    /// \brief Handler for window (re)painting.
    ///
    /// Creates and prepares a device context and calls Paint( wxDC& oDc ).
    void OnPaint( wxPaintEvent &oEvent );

    /// Repaints the image (using an internal bitmap).
    virtual void Paint( wxDC& oDc );

    /// Returns the internal bitmap.
    virtual wxBitmap& GetBitmap(){ return m_oBitmap; }

    /// Sets the internal bitmap.
    virtual void SetBitmap( const wxBitmap& oBitmap );

     /// Handle resizing.
    void OnSize( wxSizeEvent& oEvent );

    /// Handle timer events.
    void OnTimer( wxTimerEvent& oEvent );

    /// \brief Handle mouse events (non virtual part, only calls the
    ///        virtual part, see OnMouse() instead).
    ///
    /// \param oEvent       Contains the mouse event to handle.
    void OnMouse_( wxMouseEvent& oEvent ){ OnMouse( oEvent ); }

    /// \brief Handle mouse events (the virtual, customizable part).
    ///
    /// \param oEvent       Contains the mouse event to handle.
    ///
    /// OnMouse() handles the following events depending on the
    /// options set by a previous call to SetOptions():
    ///   - Scrolling while moving the mouse keeping the right mouse
    ///     button pressed (wxICANVAS_MOUSE_SCROLL).
    ///   - Scrolling vertically using the mouse wheel.
    ///     (wxICANVAS_MOUSEWHEEL_SCROLL)
    ///   - Setting a selection rectangle by using a rubber band while
    ///     keeping the left mouse button pressed.
    ///     (wxICANVAS_MOUSE_SELECTION)
    virtual void OnMouse( wxMouseEvent& oEvent );

    /// Set options for the wxIViewCanvas.
    virtual void SetOptions( int nOptions){ m_nOptions= nOptions; }

    /// Clears up (deletes) the current selection.
    virtual void ClearSelection()
    {
//        RefreshRect( m_oSelection );
        Refresh();
        m_oSelection= wxRect( -1, -1, 0, 0 );
    }

    /// Gets the selection rectangle.
    virtual wxRect& GetSelection(){ return m_oSelection; };

    /// Checks if a selection area exists.
    virtual bool IsSelected()
    {
        return    ( m_oSelection.x >= 0 )
               && ( m_oSelection.y >= 0 )
               && ( m_oSelection.width > 1 )
               && ( m_oSelection.height > 1 );
    }

  public:
    /// wxIViewCanvas options (use SetOptions() function to set).
    enum
    {
        wxICANVAS_MOUSE_SELECTION= 1,   ///< Enable selecting a rectangular image region. (Default is OFF.)
        wxICANVAS_MOUSE_SCROLL=  2,     ///< Enable scrolling following mouse cursor on right mouse key pressed. (Default is ON.)
        wxICANVAS_MOUSEWHEEL_SCROLL= 4  ///< Enable selecting a rectangular image region. (Default is ON)
    };

  protected:
    // Member variables
    int m_nOptions;             ///< Holds extended style info (options).
    wxBitmap m_oBitmap;         ///< Internal bitmap object for repainting.
    wxTimer m_oTimer;           ///< Timer for delay before sending resizing event to parent window.
    wxPoint m_oMousePosBegin;   ///< Save the starting point of a mouse movement.
    wxPoint m_oMousePosOld;     ///< Save former mouse position.
    wxRect  m_oSelection;       ///< Holds the selection.

  private:
    // Any class wishing to process wxWindows events must use this macro
    DECLARE_EVENT_TABLE()
};

#endif // __WXIVIEWCANVAS_H
